SystemOrganization addCategory: #'Math-Tests-KDTree'!

TestCase subclass: #KDTreeTest
	instanceVariableNames: 'rand stupid tree'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-KDTree'!

!KDTreeTest commentStamp: 'WernerKassens 5/3/2011 22:27' prior: 0!
KDTreeTest makes random checks of KDTree, comparing results with results of StupidNN. is not too fast.!

!KDTreeTest methodsFor: 'running' stamp: 'WernerKassens 9/15/2013 22:00'!
equalityTest: aResult and: bResult
"tests whether two collections of collections are equal"
self assert:  ( (aResult  with: bResult   collect: [:a :b|a hasEqualElements: b])  allSatisfy: [:e|e])! !

!KDTreeTest methodsFor: 'running' stamp: 'WernerKassens 9/15/2013 21:42'!
initializeTreeWith: aCollection as: simpleTree 
"used to initialize all tests"
stupid :=StupidNN withAll: aCollection.
tree := simpleTree ifTrue: [ KDTree withAll: aCollection ] ifFalse: [IndexedKDTree  withAll: aCollection ].! !

!KDTreeTest methodsFor: 'running' stamp: 'WernerKassens 11/5/2011 08:05'!
setUp
rand :=Random new.! !

!KDTreeTest methodsFor: 'tests' stamp: 'wernerkassens 5/8/2015 00:47'!
testBenchmark
|aStream s|
(RPackage  organizer includesPackageNamed: 'Math-Benchmarks-KDTree') ifFalse:[^self skip].
aStream :=ReadWriteStream with:''.
(KDTreeBenchmark fastRun: 1 with: Array)printOn: aStream . 
s :=aStream contents .
self assert: (s allRangesOfSubstring: 'Benchmark')size = 5 .
aStream :=ReadWriteStream with:''.
(KDTreeBenchmark run: 1 with: OrderedCollection )printOn: aStream . 
s :=aStream contents .
self assert:((s allRangesOfSubstring: 'Benchmark')size =7).
! !

!KDTreeTest methodsFor: 'tests' stamp: 'WernerKassens 4/15/2014 17:02'!
testDhbVectorCompatibility
|aVector bVector r|
(Array lookupSelector: #asDhbVector) ifNil:[^self  skip: 'necessary dhb package not loaded'].
aVector :=#(0)asDhbVector.
bVector :=#(-2.0)asDhbVector.
r:={aVector. #(-5.0)asDhbVector. bVector} asDhbVector .
self initializeTreeWith: r as: true .	
self assert:((tree nnSearch: bVector  i:2) ={bVector.aVector}).
self assert:((stupid nnSearch: bVector  i:2) ={bVector.aVector}).
self initializeTreeWith: r as: false .	
r:=	tree nnSearch: bVector  i:2.
self assert:((r first) =#(3 0.0) ).
self assert:(r second first =1).
self assert:(r second second closeTo: 4.0 ).! !

!KDTreeTest methodsFor: 'tests' stamp: 'WernerKassens 9/15/2013 21:51'!
testIndexedKDTree1Dimension
|m aVector aResult bResult |
m :=(rand next:200) collect:   [:n| Array with: n]."1-dimensional data, just some numbers"
self initializeTreeWith: m as: false .
1 to: 11do:[:v|aVector :={1/ v asFloat }.
	aResult :=stupid nnSearch: aVector  i:3.						"3 nearest numbers to aVector "
	bResult :=m atAll: ((tree nnSearch: aVector  i:3)collect: [:a|a first]).	
	self equalityTest: aResult and: bResult ] .! !

!KDTreeTest methodsFor: 'tests' stamp: 'WernerKassens 9/15/2013 21:53'!
testIndexedKDTree4Dimensions
|m aVector aResult bResult|
m :=(1 to: 2000) collect:   [:n| (rand next:4)-0.5].		"4-dimensional data"
self initializeTreeWith: m as: false .
50 timesRepeat: [aVector :=(rand next: 4)- 0.5.
	aResult :=stupid nnSearch: aVector  i:2.						"2 nearest neighbours"
	bResult :=m atAll: ((tree nnSearch: aVector  i:2)collect: [:a|a first]).
	self equalityTest: aResult and: bResult  ] . 
! !

!KDTreeTest methodsFor: 'tests' stamp: 'wk 2/28/2014 01:27'!
testIndexedKDTreeExtremeCase
|bResult|
tree :=IndexedKDTree  withAll: # (#( 1 )).								"extreme case"
bResult :=tree nnSearch: #(-1) i: 3.
self assert: (bResult size =1).
self assert: (bResult first = #(1 4) )    									"index 1, squared distance 4"! !

!KDTreeTest methodsFor: 'tests' stamp: 'WernerKassens 3/24/2014 13:38'!
testIndexedKDTreeSimple
|r n aTree bTree |
r :=(rand next:200) .
aTree :=IndexedKDTree withAll: (r collect: [:i| FloatArray with: i with:i])."2-dimensional data"
bTree :=IndexedKDTree withAll: (r collect: [:i| FloatArray with: i ])."1-dimensional data"
1 to: 20 do:[:v|n :=1/ v.
	self assert:((aTree nnSearch: {n.n}  i:1) =(bTree nnSearch: {n}  i:1)) ] .! !

!KDTreeTest methodsFor: 'tests' stamp: 'WernerKassens 9/15/2013 22:04'!
testKDTree1Dimension
|m aVector aResult bResult|
m :=(rand next:200) collect:   [:n| Array with: n]."1-dimensional data, just some numbers"
self initializeTreeWith: m as: true .
1 to: 11do:[:v|aVector :={1/ v asFloat }.
	aResult :=stupid nnSearch: aVector  i:3.						"3 nearest numbers to aVector "
	bResult :=tree nnSearch: aVector  i:3.							
	self equalityTest: aResult and: bResult  ]  .
! !

!KDTreeTest methodsFor: 'tests' stamp: 'wk 2/28/2014 14:12'!
testKDTree1DimensionIntegers
|m aVector aResult bResult|
m :=(1 to: 20) collect: [:index | Array with: (rand nextInt: 10)].	"only integers, obviously with multiples. test distances because the nearest neighbours are not necessarily the same"
self initializeTreeWith: m as: true .
0 to: 11do:[:v|aVector :={v}.
	aResult :=(aVector first - (stupid nnSearch: aVector  i:3))abs.		"distances between 3 nearest numbers and aVector "
	bResult :=(aVector first - (tree nnSearch: aVector  i:3))abs.		"distances using KDTree"
	self equalityTest: aResult and: bResult ]  .! !

!KDTreeTest methodsFor: 'tests' stamp: 'WernerKassens 9/15/2013 22:54'!
testKDTree2Dimensions
|aVector aResult bResult|
self initializeTreeWith: ((1 to: 2000) collect:   [:n| (rand next:2)]) as: true .	"2-dimensional data"
100 timesRepeat: [aVector :=rand next:2.
	aResult :=stupid nnSearch: aVector  i:3.
	bResult :=tree nnSearch: aVector  i:3.
	self equalityTest: aResult and: bResult  ] . ! !

!KDTreeTest methodsFor: 'tests' stamp: 'WernerKassens 2/23/2014 11:45'!
testKDTree4Dimensions
|aVector|
self initializeTreeWith: ((1 to: 2000) collect:   [:n| (rand next:4)-0.5]) as: true . 	"4-dimensional data"
50 timesRepeat: [aVector :=(rand next: 4)- 0.5.								"2 nearest neighbours"
	self equalityTest: (stupid nnSearch: aVector  i:2) and: (tree nnSearch: aVector  i:2)  ] . 
50 timesRepeat: [aVector :=(rand next: 4)-0.5.									"just 1 nearest neighbour"
	self assert: ( (stupid nnSearch: aVector  i:1)  hasEqualElements: (tree nnSearch: aVector  i:1) )    ] .  ! !

!KDTreeTest methodsFor: 'tests' stamp: 'WernerKassens 3/6/2014 15:54'!
testKDTreeExtremeCase
|bResult|

tree :=KDTree  withAll: #(#(1)).								
bResult :=tree nnSearch: #(2) i: 3.
self assert: (bResult size =1).
self assert: (bResult first = #(1) ).
self should: [ KDTree  withAll: #() ] raise: Exception.
self should: [ KDTree  withAll: #(2) ] raise: Exception.
self should: [ KDTree  withAll: #(#(2) #('v')) ] raise: Exception.
self should: [ KDTree  withAll: #(#(2) #()) ] raise: Exception.
self should: [ KDTree  withAll: #(#()) ] raise: Exception.
! !

!KDTreeTest methodsFor: 'tests' stamp: 'wernerkassens 5/8/2015 00:48'!
testPrintOn
|aStream s|
aStream :=ReadWriteStream with:''.
(KDTree withAll: #(#(1)  #(4)))printOn: aStream .
s :=aStream contents .
self assert: ((s findString: 'a KDTree')=1).
self assert: ((s allRangesOfSubstring: 'nil')size=3). "3 empty leafs should be printed"
self assert: ((s allRangesOfSubstring: '1')size=3). "twice dimension 1, once value 1"
self assert: ((s allRangesOfSubstring: '4')size=1).! !

!KDTreeTest methodsFor: 'tests' stamp: 'wernerkassens 5/8/2015 00:48'!
testPrintOn2
|aStream s|
aStream :=ReadWriteStream with:''.
(IndexedKDTree withAll: #(#(1)))printOn: aStream.
s :=aStream contents .
self assert: ((s findString: 'an IndexedKDTree')=1).
self assert: ((s allRangesOfSubstring: '1')size=2).! !

TestCase subclass: #NNStoreTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-KDTree'!

!NNStoreTest commentStamp: 'wk 2/28/2014 22:36' prior: 0!
the individual tests also check and show the general way NNStore processes its data.
!

!NNStoreTest methodsFor: 'tests' stamp: 'WernerKassens 3/5/2014 17:50'!
testCopy
"a separate copy method is necessary because of this scenario"
|a b|
a :=NNStore withAll: #(1 2 4).
b:=a copy.
self assert: (b=a).
self assert: (b hash=a hash).
b sortFor: nil.
self deny: (b=a).	"here is the problem"
self deny: (b hash=a hash).! !

!NNStoreTest methodsFor: 'tests' stamp: 'WernerKassens 3/9/2014 09:46'!
testCopyEmpty
|n m|
n :=NNStore newFrom:#(#(1 $a) #(3 1.0) #(0 '2')).
n:=n copyEmpty .
self assert: n isEmpty .
n add: #(2 10); add: #(1 100).
self deny: n isEmpty .
self assert: (n maxDistance=Float infinity). 	"because it is not yet full"
self assert: (n data =#(100 10)).
m:=n copyEmpty.
self assert: (m size =2). "size is reduced now"
m add: #(2 10); add: #(1 100).
self deny:(m=n).
self assert:(m completeData =n completeData ).! !

!NNStoreTest methodsFor: 'tests' stamp: 'WernerKassens 3/9/2014 10:02'!
testEqual
|a b|
b :=#(#(1 $a) #(3 1.0) #(0 '2')).
a :=NNStore newFrom: b.
self deny: (a=b).
b :=NNStore newFrom:#(#(1 $a) #(3 1.0) #(0 2)).
self deny: (a=b).
b :=a copyEmpty .
self deny: (a=b).
b addAll: #(#(3 1.0) #(1 $a) #(4 true) #(0 '2')).
self assert: (a=b).
b add: #(2 nil).
self deny: (a=b).
b :=NNStore new:3.
a :=NNStore new:2.
a add: #(2 nil).
b add: #(2 nil).
self deny: (a=b).
self deny: (b hash=a hash).! !

!NNStoreTest methodsFor: 'tests' stamp: 'wernerkassens 10/27/2014 16:38'!
testExtremeCase
|n|
n:=NNStore new.
n sortFor: nil. 	"this should indeed always be possible and shouldnt raise an error!!"
self assert: n isFull .
n add: #(1 1).	"adding should also always be possible, although it will not be added in this case"
self deny: (n maxDistance=1).
n add: {0-Float infinity.1}. "and this extreme case too."
self assert: n isEmpty .
self assert: (n data =#()).
self should: [n add: {nil .1}] raise: Error . "but this should always raise an error (not only in NNStores with size 0), also in subclasses, otherwise one can have strange bugs"


! !

!NNStoreTest methodsFor: 'tests' stamp: 'WernerKassens 3/4/2014 17:18'!
testInitialize
|a b|
a :=(NNStore newFrom:#(#(4 $a) #(5 1.0) #(6 '2')))  .	
b :=(NNStore newFrom:#(#(4 $a) #(5 1.0) #(6 '2'))) initialize .	
self deny: (a=b).
a initialize.
self assert: (a=b).
b add: #(4 $a). 
self deny: (a=b).
a add:#(4 $a).
self assert: (a=b).	
a add: #(5 1.0). 
self deny: (a=b).
! !

!NNStoreTest methodsFor: 'tests' stamp: 'WernerKassens 3/4/2014 17:20'!
testInitialize2
"a separate = method is necessary because of this scenario with resetting"
|a b c|
a :=(NNStore newFrom:#(#(1 $a) #(3 1.0) #(0 '2'))) initialize .	"initialize = resetting"
b :=(NNStore newFrom:#(#(1 $a) #(3 false) #(0 '2'))) initialize .	
self assert: (a=b).	"first  problematic part"
c:=#(#(3 true) #(2 1)).
a addAll: c. b addAll: c.
self assert: (a=b).	"second problematic part"
c:=#(2 nil).
a add: c. 
self deny: (a=b).
b add: c.
self assert: (a=b).
! !

!NNStoreTest methodsFor: 'tests' stamp: 'WernerKassens 3/4/2014 17:27'!
testInitialize3
|a|
a :=(NNStore newFrom:#(#(7 $a) #(5 1.0) #(6 '2'))) initialize .	
a add: #(5 1.0); add: #(4 $a). 
self assert: (a data=#($a 1.0)).
self assert: (a completeData =#(#(4 $a) #(5 1.0)) ).	


! !

!NNStoreTest methodsFor: 'tests' stamp: 'WernerKassens 3/3/2014 12:31'!
testNew
|n|
n:=NNStore new:1.
self assert: n isEmpty .
self assert: (n data =#()).
self assert: (n completeData  =#()).
n add: #(1 2).
n add: #(3 1). "will not be added"
self assert: (n data =#(2)).
! !

!NNStoreTest methodsFor: 'tests' stamp: 'WernerKassens 3/3/2014 12:32'!
testNew2
|n|
n:=NNStore new:3.
n add: #(3 1).
n add: #(1 2).
self deny: n isFull  .
self assert: (n completeData =#(#(1 2) #(3 1)) )! !

!NNStoreTest methodsFor: 'tests' stamp: 'WernerKassens 3/3/2014 12:32'!
testNew3
|n|
n:=NNStore new:0.
self assert: n isEmpty .
self assert: n isFull .
self assert: (n =n copyEmpty)  .
self assert: (n data =#()).
self assert: (n completeData  =#()).
! !

!NNStoreTest methodsFor: 'tests' stamp: 'WernerKassens 3/3/2014 12:33'!
testNewFrom
|n |
n :=NNStore newFrom:#(#(1 $a) #(3 1.0) #(0 '2')).
self assert: (n  data= #('2' $a 1.0) ).
self assert: n isFull .
self assert: (n maxDistance=3).
n add: #(2 true).
self assert: (n maxDistance=2).
n add: #(0.5 nil).
self assert: (n maxDistance=1).
self assert: (n data = #('2' nil $a)).
! !

!NNStoreTest methodsFor: 'tests' stamp: 'WernerKassens 3/5/2014 21:07'!
testWithAll
|n| 
n :=NNStore withAll: #(2 5 4).
self assert: (n completeData =#(#(nil 2) #(nil 5) #(nil 4)) ).
n sortFor: nil.	"calling sortFor: is necessary with withAll:"
self assert: (n data =#(2 4 5)).
self assert: n isFull  .
n add: {3 . 6}. 	"one has to be a bit carefull when one uses 
				 NNStore withAll:, as sortfor: changes this data. 
				 n add: #(3 6) is not possible here!!"
self assert: (n data =#(2 6 4)).
n sortFor: nil.
self assert: (n data =#(2 4 6)).
! !
